import "reflect-metadata";
import {createTestingConnections, closeTestingConnections, reloadTestingDatabases} from "../../utils/test-utils";
import {Connection} from "../../../src/connection/Connection";
import {Post} from "./entity/Post";
import {expect} from "chai";
import {Category} from "./entity/Category";

describe("other issues > using limit in conjunction with order by", () => {

    let connections: Connection[];
    before(async () => connections = await createTestingConnections({
        entities: [__dirname + "/entity/*{.js,.ts}"],
        schemaCreate: true,
        dropSchemaOnConnection: true,
    }));
    beforeEach(() => reloadTestingDatabases(connections));
    after(() => closeTestingConnections(connections));

    it("should persist successfully and return persisted entity", () => Promise.all(connections.map(async function(connection) {

        // generate bulk array of posts with categories
        const promises: Promise<any>[] = [];
        for (let i = 1; i <= 100; i++) {

            const post = new Post();
            post.title = "Hello Post #" + i;
            post.categories = [];

            for (let i = 1; i <= 5; i++) {
                const category = new Category();
                category.name = "category #" + i;
                post.categories.push(category);
            }
            promises.push(connection.entityManager.persist(post));
        }

        await Promise.all(promises);

        // check if ordering by main object works correctly

        const loadedPosts1 = await connection.entityManager
            .createQueryBuilder(Post, "post")
            .innerJoinAndSelect("post.categories", "categories")
            .setMaxResults(10)
            .orderBy("post.id", "DESC")
            .getMany();

        expect(loadedPosts1).not.to.be.empty;
        loadedPosts1.length.should.be.equal(10);
        loadedPosts1[0].id.should.be.equal(100);
        loadedPosts1[1].id.should.be.equal(99);
        loadedPosts1[2].id.should.be.equal(98);
        loadedPosts1[3].id.should.be.equal(97);
        loadedPosts1[4].id.should.be.equal(96);
        loadedPosts1[5].id.should.be.equal(95);
        loadedPosts1[6].id.should.be.equal(94);
        loadedPosts1[7].id.should.be.equal(93);
        loadedPosts1[8].id.should.be.equal(92);
        loadedPosts1[9].id.should.be.equal(91);

    })));

});